@@ -2,9 +2,10 @@ import re
 from configparser import ConfigParser
 from inspect import cleandoc
 
+import jaraco.path
 import pytest
 import tomli_w
-from path import Path as _Path
+from path import Path
 
 from setuptools.config.pyprojecttoml import (
     read_configuration,
@@ -82,25 +83,32 @@ universal = true
 
 
 def create_example(path, pkg_root):
-    pyproject = path / "pyproject.toml"
+    files = {
+        "pyproject.toml": EXAMPLE,
+        "README.md": "hello world",
+        "_files": {
+            "file.txt": "",
+        },
+    }
+    packages = {
+        "pkg": {
+            "__init__.py": "",
+            "mod.py": "class CustomSdist: pass",
+            "__version__.py": "VERSION = (3, 10)",
+            "__main__.py": "def exec(): print('hello')",
+        },
+    }
 
-    files = [
-        f"{pkg_root}/pkg/__init__.py",
-        "_files/file.txt",
-    ]
-    if pkg_root != ".":  # flat-layout will raise error for multi-package dist
-        # Ensure namespaces are discovered
-        files.append(f"{pkg_root}/other/nested/__init__.py")
+    assert pkg_root  # Meta-test: cannot be empty string.
 
-    for file in files:
-        (path / file).parent.mkdir(exist_ok=True, parents=True)
-        (path / file).touch()
+    if pkg_root == ".":
+        files = {**files, **packages}
+        # skip other files: flat-layout will raise error for multi-package dist
+    else:
+        # Use this opportunity to ensure namespaces are discovered
+        files[pkg_root] = {**packages, "other": {"nested": {"__init__.py": ""}}}
 
-    pyproject.write_text(EXAMPLE)
-    (path / "README.md").write_text("hello world")
-    (path / f"{pkg_root}/pkg/mod.py").write_text("class CustomSdist: pass")
-    (path / f"{pkg_root}/pkg/__version__.py").write_text("VERSION = (3, 10)")
-    (path / f"{pkg_root}/pkg/__main__.py").write_text("def exec(): print('hello')")
+    jaraco.path.build(files, prefix=path)
 
 
 def verify_example(config, path, pkg_root):
@@ -174,7 +182,7 @@ class TestEntryPoints:
     def write_entry_points(self, tmp_path):
         entry_points = ConfigParser()
         entry_points.read_dict(ENTRY_POINTS)
-        with open(tmp_path / "entry-points.txt", "w") as f:
+        with open(tmp_path / "entry-points.txt", "w", encoding="utf-8") as f:
             entry_points.write(f)
 
     def pyproject(self, dynamic=None):
@@ -208,11 +216,13 @@ class TestClassifiers:
         # Let's create a project example that has dynamic classifiers
         # coming from a txt file.
         create_example(tmp_path, "src")
-        classifiers = """\
-        Framework :: Flask
-        Programming Language :: Haskell
-        """
-        (tmp_path / "classifiers.txt").write_text(cleandoc(classifiers))
+        classifiers = cleandoc(
+            """
+            Framework :: Flask
+            Programming Language :: Haskell
+            """
+        )
+        (tmp_path / "classifiers.txt").write_text(classifiers, encoding="utf-8")
 
         pyproject = tmp_path / "pyproject.toml"
         config = read_configuration(pyproject, expand=False)
@@ -240,7 +250,7 @@ class TestClassifiers:
         """
 
         pyproject = tmp_path / "pyproject.toml"
-        pyproject.write_text(cleandoc(config))
+        pyproject.write_text(cleandoc(config), encoding="utf-8")
         with pytest.raises(OptionError, match="No configuration .* .classifiers."):
             read_configuration(pyproject)
 
@@ -252,7 +262,7 @@ class TestClassifiers:
         dynamic = ["readme"]
         """
         pyproject = tmp_path / "pyproject.toml"
-        pyproject.write_text(cleandoc(config))
+        pyproject.write_text(cleandoc(config), encoding="utf-8")
         dist = Distribution(attrs={"long_description": "42"})
         # No error should occur because of missing `readme`
         dist = apply_configuration(dist, pyproject)
@@ -270,7 +280,7 @@ class TestClassifiers:
         """
 
         pyproject = tmp_path / "pyproject.toml"
-        pyproject.write_text(cleandoc(config))
+        pyproject.write_text(cleandoc(config), encoding="utf-8")
         with pytest.warns(UserWarning, match="File .*classifiers.txt. cannot be found"):
             expanded = read_configuration(pyproject)
         assert "classifiers" not in expanded["project"]
@@ -291,7 +301,7 @@ class TestClassifiers:
 )
 def test_ignore_unrelated_config(tmp_path, example):
     pyproject = tmp_path / "pyproject.toml"
-    pyproject.write_text(cleandoc(example))
+    pyproject.write_text(cleandoc(example), encoding="utf-8")
 
     # Make sure no error is raised due to 3rd party configs in pyproject.toml
     assert read_configuration(pyproject) is not None
@@ -313,7 +323,7 @@ def test_ignore_unrelated_config(tmp_path, example):
 )
 def test_invalid_example(tmp_path, example, error_msg):
     pyproject = tmp_path / "pyproject.toml"
-    pyproject.write_text(cleandoc(example))
+    pyproject.write_text(cleandoc(example), encoding="utf-8")
 
     pattern = re.compile(f"invalid pyproject.toml.*{error_msg}.*", re.M | re.S)
     with pytest.raises(ValueError, match=pattern):
@@ -323,7 +333,7 @@ def test_invalid_example(tmp_path, example, error_msg):
 @pytest.mark.parametrize("config", ("", "[tool.something]\nvalue = 42"))
 def test_empty(tmp_path, config):
     pyproject = tmp_path / "pyproject.toml"
-    pyproject.write_text(config)
+    pyproject.write_text(config, encoding="utf-8")
 
     # Make sure no error is raised
     assert read_configuration(pyproject) == {}
@@ -335,7 +345,7 @@ def test_include_package_data_by_default(tmp_path, config):
     default.
     """
     pyproject = tmp_path / "pyproject.toml"
-    pyproject.write_text(config)
+    pyproject.write_text(config, encoding="utf-8")
 
     config = read_configuration(pyproject)
     assert config["tool"]["setuptools"]["include-package-data"] is True
@@ -347,12 +357,13 @@ def test_include_package_data_in_setuppy(tmp_path):
 
     See https://github.com/pypa/setuptools/issues/3197#issuecomment-1079023889
     """
-    pyproject = tmp_path / "pyproject.toml"
-    pyproject.write_text("[project]\nname = 'myproj'\nversion='42'\n")
-    setuppy = tmp_path / "setup.py"
-    setuppy.write_text("__import__('setuptools').setup(include_package_data=False)")
+    files = {
+        "pyproject.toml": "[project]\nname = 'myproj'\nversion='42'\n",
+        "setup.py": "__import__('setuptools').setup(include_package_data=False)",
+    }
+    jaraco.path.build(files, prefix=tmp_path)
 
-    with _Path(tmp_path):
+    with Path(tmp_path):
         dist = distutils.core.run_setup("setup.py", {}, stop_after="config")
 
     assert dist.get_name() == "myproj"
